'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isFlowType = isFlowType;
exports.getType = getType;
exports.getDefaultValue = getDefaultValue;
exports.getKey = getKey;
exports.getTypeName = getTypeName;
exports.isComplexType = isComplexType;
exports.blockquote = blockquote;
exports.filterProps = filterProps;

var _kebabCase = require('lodash/fp/kebabCase');

var _kebabCase2 = _interopRequireDefault(_kebabCase);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function isFlowType(prop) {
  return !!prop.flowType;
}

function getType(prop) {
  return prop.type || prop.flowType;
}

function getDefaultValue(prop) {
  var value = prop.defaultValue;
  var type = getType(prop).name;
  if (!value) {
    switch (type) {
      case 'bool':
      case 'boolean':
        return false; // Default for boolean is false
      default:
        return '';
    }
  }

  if (typeof value.value !== 'undefined' && value.value) {
    return value.value;
  }
  return value;
}

function getKey(key, type) {
  if (isComplexType(type.name)) {
    // TODO: Add support for Github markdown?
    // Add Bitbucket header link
    return '[' + key + '](#markdown-header-' + (0, _kebabCase2.default)(key) + ')';
  }

  return key;
}

/**
 * Transform type into correct string
 * @param type
 * @returns {*}
 */
function getTypeName(type) {
  if (type.raw) return formatCustomType(type.raw);

  switch (type.name) {
    case 'union':
    case 'literalsAndUnion':
      return 'Union';
    case 'shape':
    case 'signature':
      return 'Object';
    default:
      return type.name;
  }
}

function isComplexType(name) {
  switch (name) {
    case 'union':
    case 'literalsAndUnion':
    case 'shape':
    case 'arrayOf':
    case 'Array':
    case 'enum':
    case 'signature':
      return true;
    default:
      return false;
  }
}

function blockquote(input) {
  return input.replace(/\n/g, '\n> ');
}

function filterProps(name, prop, _ref) {
  var excludeKey = _ref.excludeKey,
      excludeType = _ref.excludeType,
      excludeDescription = _ref.excludeDescription;

  if (!getType(prop)) {
    // eslint-disable-next-line no-console
    console.error('Found prop \'' + name + '\' without type. Has it been removed, but left in \'defaultProps\'?');
  }
  if (excludeKey && excludeKey.test(name)) return false;
  var type = getType(prop);

  if (excludeType && type && excludeType.test(type.name)) return false;
  if (excludeDescription && excludeDescription.test(prop.description)) return false;

  return true;
}

function formatCustomType(name) {
  return name
  // If the name has the ".isRequired", remove it. Most likely a PropTypes.shape() object that's required.
  // Docgen will pick it up, and mark the value as required
  .replace('.isRequired', '').replace('|', '|');
}